/*
 * Tara Graeve
 * Steganography Project 
 * 5/20/19
 */

package steganography;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;

public class Steganography {

	public static int[] getDataLocations(int start, int stop, int numLocations) throws NotEnoughSpaceException {
		int fileSize; 
		int distanceApart; 

		fileSize = stop - start; 

		if (numLocations*6 > fileSize) { // int+char = 6 bytes, so if numLocations*6 is greater than the file size, there's not enough space to store the message
			throw new NotEnoughSpaceException(numLocations*6,fileSize);
		}

		distanceApart = fileSize/numLocations; // calculate how far apart characters need to be 

		int[] locations = new int[numLocations]; 

		for(int i = 0; i < numLocations; i++) {
			locations[i] = i*distanceApart + start; // fill array with correct locations
		}

		return locations; 
	}

	public static String decodeMessage(File inputFile) throws SecretMessageException {
		String message = null; 
		try {

			RandomAccessFile raf = new RandomAccessFile(inputFile,"r"); 
			raf.seek(50); // skip past header

			int messageLength = raf.readInt(); 
			raf.seek(54);
			char[] chars = new char[messageLength]; // array of chars that represents the message 

			for (int i = 0; i < chars.length; i++) {
				if(i < chars.length-1) { // if it's not the end, read char then seek for next int 
					chars[i] = raf.readChar();
					raf.seek(raf.readInt());
				} else { // if this point is reached, it's the end of the message, so just read the char, don't seek 
					chars[i] = raf.readChar();
				}				
			}

			message = new String(chars); // convert array of chars to a string using String constructor that takes a char array 

			try {
				raf.close(); // try to close RandomAccessFile
			} catch (IOException e) {
				throw new SecretMessageException("Could  not close file."); // catch and display this message if closing unsuccessful 
			}

		} catch (FileNotFoundException e) {
			throw new SecretMessageException("File not found."); // catch if computer couldn't find file 
		} catch (IOException e) {
			throw new SecretMessageException("Could not read file."); // catch if computer couldn't read file 
		}

		return message; 

	}

	public static void encodeMessage(File inputFile, File outputFile, String message) throws Exception {
		int[] locations = null; 
		try {
			Files.copy(inputFile.toPath(), outputFile.toPath(), StandardCopyOption.REPLACE_EXISTING); // makes copy of input file and puts it in output file 
			RandomAccessFile raf = new RandomAccessFile(outputFile,"rw");
			raf.seek(50); // skip header 
			raf.writeInt(message.length()); 

			try {
				locations = getDataLocations(54, ((int)raf.length()), message.length()); // set array equal to array of message locations

			} catch (NotEnoughSpaceException e) { // catch if not enough space 
				System.out.println(e.getMessage());
			}

			for(int i = 0; i < message.length(); i++) { 
				raf.seek(locations[i]); // seek to location of hidden char
				raf.writeChar(message.charAt(i)); // write the char that's at that index

				if(i == message.length()-1) { // go back to the beginning of it's the end of the message 
					raf.writeInt(0);
				} else { // write int of next index if it's not the end 
					raf.writeInt(locations[i+1]);
				}


			}
			try { 
				raf.close();
			} catch (IOException e) {
				System.out.println("Could not close file.");
			}

		} catch (Exception e) {
			throw e; // re throw "e" to pass the buck on errors 

		}

	}



}
